/*
oop_ex101.cpp
friend functions in inheritance
*/

#include <iostream>
#include <math.h>

using namespace std;

// The point base class
class point
{
	double x;
	double y;
public:
	point();
	point(double xx, double yy);
	point(const point& other);
	point& operator=(const point& other);
	~point() { cout << "~point() for x = " << x << " y = " << y << endl; }

	double getX() { return x; }
	double getY() { return y; }

	void moveTo(double xx, double yy);
	double distTo(double xx, double yy);
	void print();

	//friend function
	friend void doubleXY(point& p);
};


point::point()
{
	cout << "point::point()" << endl;
	x = 0.0;
	y = 0.0;
}

point::point(double xx, double yy)
{
	cout << "point::point(double xx, double yy)" << endl;
	x = xx;
	y = yy;
}


point::point(const point& other)
{
	cout << "point::point(const point& other)" << endl;
	x = other.x;
	y = other.y;
}

point& point::operator=(const point& other)
{
	cout << "point& point::operator=(const point& other)" << endl;
	x = other.x;
	y = other.y;

	return *this;
}


void point::moveTo(double xx, double yy)
{
	x = xx;
	y = yy;
}

double point::distTo(double xx, double yy)
{
	double dx = x - xx;
	double dy = y - yy;

	return sqrt(dx*dx + dy * dy);
}

void point::print()
{
	cout << "x = " << x << ", ";
	cout << "y = " << y << endl;
}


// The circle derived class
class circle : public point
{
	double radius;
public:
	circle();
	circle(double r);
	circle(double xx, double yy, double r);
	circle(const circle& other);
	circle& operator=(const circle& other);
	~circle() { cout << "~circle() for radius = " << radius << endl; }

	// function to be modified, need to be listed here
	void print();

	// new function members
	double area();
	double perimeter();
	void setR(double r) { radius = r; }
	double getR() { return radius; }
};


// constructors
circle::circle() : point()	//call a constructor of base class
{
	cout << "circle::circle()" << endl;
	radius = 0.0;
}

circle::circle(double r) : point()
{
	cout << "circle::circle(double r)" << endl;
	radius = r;
}

circle::circle(double xx, double yy, double r) : point(xx, yy)
{
	cout << "circle::circle(double xx, double yy, double r)" << endl;
	radius = r;
}

circle::circle(const circle& other) : point(other)
{
	cout << "circle::circle(const circle& other)" << endl;
	radius = other.radius;
}

circle& circle::operator=(const circle& other)
{
	cout << "circle& circle::operator=(const circle& other)" << endl;

	point::operator=(other);
	radius = other.radius;

	return *this;
}


// function to be overridden
void circle::print()
{
	point::print();

	cout << "radius = " << radius << " ";
	cout << "area = " << area() << " ";
	cout << "perimeter = " << perimeter() << endl;
}


// new function members
double circle::area()
{
	return 3.14159*radius*radius;
}

double circle::perimeter()
{
	return 2 * 3.14159*radius;
}



//friend function
void doubleXY(point& p)
{
	p.x *= 2;
	p.y *= 2;
	//p.radius*=2;
}



void main()
{
	point p1(3, 4);
	doubleXY(p1);

	circle c1(30, 40, 5);
	doubleXY(c1);

	cout << "-------------------" << endl;
	p1.print();
	cout << "-------------------" << endl;
	c1.print();
	cout << "-------------------" << endl;
	
	system("pause");
}


