/*
oop_ex93.cpp
inheritance
*/

#include <iostream>
#include <math.h>

using namespace std;

// The point base class
class point
{
	double x;
	double y;
	//void print();		//Can override private function member?
public:
	point();
	point(double xx, double yy);
	~point() { cout << "~point() for x = " << x << " y = " << y << endl; }

	void moveTo(double xx, double yy);
	double distTo(double xx, double yy);
	double getX() { return x; }
	double getY() { return y; }
	void print();
};

point::point()
{
	cout << "point::point()" << endl;
	x = 0.0;
	y = 0.0;
}

point::point(double xx, double yy)
{
	cout << "point::point(double xx, double yy)" << endl;
	x = xx;
	y = yy;
}

void point::moveTo(double xx, double yy)
{
	x = xx;
	y = yy;
}

double point::distTo(double xx, double yy)
{
	double dx = x - xx;
	double dy = y - yy;

	return sqrt(dx*dx + dy * dy);
}

void point::print()
{
	cout << "point::print()" << endl;
	cout << "x = " << x << ", ";
	cout << "y = " << y << endl;
}


// The circle derived class
class circle : public point
{
	double radius;
public:
	circle();
	circle(double r);
	circle(double xx, double yy, double r);
	~circle() { cout << "~circle() for radius = " << radius << endl; };

	// function to be overridden, need to be listed here
	void print();

	// new function members
	double area();
	double perimeter();
	void setR(double r) { radius = r; }
	double getR() { return radius; }
};


// constructors
circle::circle() : point()	//call a constructor of base class
{
	cout << "circle::circle()" << endl;
	//x = 0.0;   //can't access private members of base class
	//y = 0.0;
	radius = 0.0;
}

circle::circle(double r) : point()
{
	cout << "circle::circle(double r)" << endl;
	radius = r;
}

circle::circle(double xx, double yy, double r) : point(xx, yy)
{
	cout << "circle::circle(double xx, double yy, double r)" << endl;
	radius = r;
}


// function to be modified/overridden
void circle::print()
{
	cout << "circle::print()" << endl;

	//1. NG, can't access private members of base class
	//cout<<"x = "<<x<<", ";
	//cout<<"y = "<<y<<endl;

	//2. OK, use public access functions in base class
	cout << "x = " << getX() << ", ";
	cout << "y = " << getY() << endl;

	//3. OK, calling the original function in base class
	//point::print();

	cout << "radius = " << radius << " ";
	cout << "area = " << area() << " ";
	cout << "perimeter = " << perimeter() << endl;
}


// new function members
double circle::area()
{
	return 3.14159*radius*radius;
}

double circle::perimeter()
{
	return 2 * 3.14159*radius;
}





void main()
{
	point p1(3, 4);
	circle c1(30, 40, 5);

	cout << "p1.print();" << endl;
	p1.print();

	cout << endl << "c1.print();" << endl;
	c1.print();

	c1.moveTo(15, 20);

	cout << endl << "c1.print();" << endl;
	c1.print();
	
	circle* c2 = new circle(30, 40, 5);
	delete c2;

	system("pause");
}
