/*
cpp_ex95.cpp
Casting Base-Class Pointers to Derived-Class Pointers
*/

#include <iostream>
#include <math.h>

using namespace std;

// The point base class
class point
{
	double x;
	double y;
public:
	point();
	point(double xx, double yy);
	~point() { /*cout<<"~point() for x = "<<x<<" y = "<<y<<endl;*/ }

	void moveTo(double xx, double yy);
	double distTo(double xx, double yy);
	//double getX() { return x; }
	//double getY() { return y; }
	void print();
};

point::point()
{
	//cout<<"point::point()"<<endl;
	x = 0.0;
	y = 0.0;
}

point::point(double xx, double yy)
{
	//cout<<"point::point(double xx, double yy)"<<endl;
	x = xx;
	y = yy;
}

void point::moveTo(double xx, double yy)
{
	x = xx;
	y = yy;
}

double point::distTo(double xx, double yy)
{
	double dx = x - xx;
	double dy = y - yy;

	return sqrt(dx*dx + dy * dy);
}

void point::print()
{
	cout << "x = " << x << ", ";
	cout << "y = " << y << endl;
}


// The circle derived class
class circle : public point
{
	double radius;
public:
	circle();
	circle(double r);
	circle(double xx, double yy, double r);
	~circle() { /*cout<<"~circle() for radius = "<< radius <<endl;*/ }

	// function to be overridden
	void print();

	// new function members
	double area();
	double perimeter();
	void setR(double r) { radius = r; }
	double getR() { return radius; }
};


// constructors
circle::circle() : point()	//call a constructor of base class
{
	//cout<<"circle::circle()"<<endl;
	radius = 0.0;
}

circle::circle(double r) : point()
{
	//cout<<"circle::circle(double r)"<<endl;
	radius = r;
}

circle::circle(double xx, double yy, double r) : point(xx, yy)
{
	//cout<<"circle::circle(double xx, double yy, double r)"<<endl;
	radius = r;
}


// function to be overridden
void circle::print()
{
	point::print();

	cout << "radius = " << radius << " ";
	cout << "area = " << area() << " ";
	cout << "perimeter = " << perimeter() << endl;
}


// new function members
double circle::area()
{
	return 3.14159*radius*radius;
}

double circle::perimeter()
{
	return 2 * 3.14159*radius;
}





void main()
{
	point p1(3, 4);
	circle c1(30, 40, 5);


	point* pptr1;
	circle* cptr1;

	cout << "--- circle is a point ---" << endl;

	point& pref1 = c1;	// assign object of derived class to reference of base class, OK
	pptr1 = &c1;			// assign object of derived class to pointer of base class, OK

	cout << "pptr1->print();" << endl;
	pptr1->print();

	cptr1 = (circle*)pptr1; // downcasting, the object pptr1 points to (c1) is a circle, OK
	cout << "cptr1->print();" << endl;
	cptr1->print();


	//-----------------------------------------------------------------
	cout << "--- point is not a circle ---" << endl;


	cptr1 = (circle*)&p1;	// downcasting, p1 is not a circle, allowed but NG
	//cptr1 = static_cast<circle*>(&p1); // another format
	cout << "cptr1->print();" << endl;
	cptr1->print();
	
	system("pause");
}

